@@ -92,7 +92,66 @@ const char* VTZ_OFROM = "TZOFFSETFROM";
 const char* VTZ_OTO   = "TZOFFSETTO";
 const char* VTZ_NAME  = "TZNAME";
 
+/*!
+  * escapes newline ('\n'), comma, semicolon and backslash itself with backslash,
+  * according to RFC 2445 "4.3.1 Text" value definition
+  */
+static string escapeText(const string &str)
+{
+  string res;
+  res.reserve(str.size() * 110 / 100);
 
+  for (size_t i = 0; i < str.size(); i++) {
+    switch (str[i]) {
+    case '\n':
+      res += "\\n";
+      break;
+    case '\\':
+    case ',':
+    case ';':
+      res += '\\';
+      // no break!
+    default:
+      res += str[i];
+      break;
+    }
+  }
+
+  return res;
+}
+
+/*!
+ * reverses escapeText(); in addition, accepts arbitrary characters
+ * after backslash and replaces them with that character
+ */
+static string unescapeText(const string &str)
+{
+  string res;
+  res.reserve(str.size());
+
+  for (size_t i = 0; i < str.size(); i++) {
+    switch (str[i]) {
+    case '\\':
+      i++;
+      if (i < str.size()) {
+        switch (str[i]) {
+        case 'n':
+          res += '\n';
+          break;
+        default:
+          res += str[i];
+          break;
+        }
+      }
+      break;
+    default:
+      res += str[i];
+      break;
+    }
+  }
+
+  return res;
+}
 
 /*! RRULE2toInternal, using RType
  *  Converts  vCalendar 2.0 RRULE string into internal recurrence representation
@@ -290,7 +349,7 @@ static bool GetTZInfo( cAppCharP     aText,
          st= VValue( a, VTZ_START ); //            - start time
   string of= VValue( a, VTZ_OFROM ); //            - tz offset from
   string ot= VValue( a, VTZ_OTO   ); //            - tz offset to
-      cName= VValue( a, VTZ_NAME  ); //            - tz name
+      cName= unescapeText( VValue( a, VTZ_NAME  ) ); // - tz name
 
   if (ISO8601StrToTimestamp( st.c_str(), dtstart, tctx )==0)  return false;
   if (!Get_Bias( of,ot, cBias ))                              return false;
@@ -411,8 +470,6 @@ static void MultipleSeq( string aText, int &s, int &d )
 
 bool VTIMEZONEtoTZEntry( const char*    aText, // VTIMEZONE string to be parsed
                          tz_entry      &t,
-                         string        &aStdName,
-                         string        &aDstName,
                          TDebugLogger*  aLogP)
 {
   short dBias; // the full bias for DST
@@ -423,12 +480,14 @@ bool VTIMEZONEtoTZEntry( const char*    aText, // VTIMEZONE string to be parsed
                                  t.dynYear= "CUR";
                                  t.biasDST= 0;
                                  t.bias   = 0;
-  if (!GetTZInfo( aText,VTZ_STD, t.std, t.bias, aStdName, -1, aLogP )) {
+                                 t.stdName =
+                                   t.dstName = "";
+  if (!GetTZInfo( aText,VTZ_STD, t.std, t.bias, t.stdName, -1, aLogP )) {
     success = false;
   }
   // default value if not found (which is treated as success by GetTZInfo)
   dBias= t.bias;
-  if (!GetTZInfo( aText,VTZ_DST, t.dst,  dBias, aDstName, -1, aLogP )) {
+  if (!GetTZInfo( aText,VTZ_DST, t.dst,  dBias, t.dstName, -1, aLogP )) {
     // unknown failure, better restore default
     dBias= t.bias;
     success = false;
@@ -438,7 +497,7 @@ bool VTIMEZONEtoTZEntry( const char*    aText, // VTIMEZONE string to be parsed
   else t.biasDST= dBias - t.bias; // t.biasDST WILL be calculated here
 
   // get TZID as found in VTIMEZONE
-  t.name = VValue( aText, VTZ_ID );
+  t.name = unescapeText( VValue( aText, VTZ_ID ) );
 
   return success;
 } // VTIMEZONEtoTZEntry
@@ -454,25 +513,19 @@ bool VTIMEZONEtoInternal( const char*    aText, // VTIMEZONE string to be parsed
   aContext= tctx_tz_unknown;
 
   tz_entry      t;
-  string        stdName,
-                dstName,
-                lName;
+  string        lName;
   timecontext_t lContext;
 
-  if (!VTIMEZONEtoTZEntry( aText, t, stdName, dstName, aLogP )) {
+  if (!VTIMEZONEtoTZEntry( aText, t, aLogP )) {
     PLOGDEBUGPRINTFX(aLogP, DBG_PARSE+DBG_ERROR, ("parsing VTIMEZONE failed:\n%s", aText));
   }
+  // Telling the caller about the original TZID is necessary because this
+  // code might match that TZID against an existing definition with a different
+  // TZID. Previously it was also necessary when importing the VTIMEZONE, because
+  // the original TZID was overwritten. Now imported definitions retain the
+  // original TZID in t.name.
   if (aTzidP) *aTzidP = t.name; // return the original TZID as found, needed to match with TZID occurences in rest of vCalendar
 
-  bool sC= !(stdName=="");
-  bool dC= !(dstName=="");
-
-  string tName = t.name;
-  if  (sC || dC) tName = ""; // TZID will be replaced in case of unknown
-  if  (sC)       tName+= stdName;
-  if  (sC && dC) tName+= "/";
-  if        (dC) tName+= dstName;
-
   bool ok = true;
   bool okM= true;
   int                 s,d;
@@ -509,19 +562,19 @@ bool VTIMEZONEtoInternal( const char*    aText, // VTIMEZONE string to be parsed
   // redundant here, but there is no other way to
   // add the entry
   string new_name;
-  t.name = tName;
+
   if (!ok) ok= FoundTZ( t, new_name, aContext, g, true );
 
   if  (ok && t.std.wMonth!=0 &&
              t.dst.wMonth!=0) {
     tz_entry std;
-    std.name = stdName;
+    std.name = t.stdName;
     std.ident= "s"; // standard
     std.bias = t.bias;
     FoundTZ( std, lName,lContext, g, true );
 
     tz_entry dst;
-    dst.name = dstName;
+    dst.name = t.dstName;
     dst.ident= "d"; // daylight saving
     dst.bias = t.bias + t.biasDST;
     FoundTZ( dst, lName,lContext, g, true );
@@ -607,7 +660,7 @@ static string GenerateTZInfo( tz_entry t,
   if (withDST) s+= Property( VTZ_RR,    rTxt      );
                s+= FromTo             ( aFrom,aTo );
   if (withDST &&
-     !aName.empty()) s+= Property( VTZ_NAME, aName );
+     !aName.empty()) s+= Property( VTZ_NAME, escapeText( aName ) );
 
   return Encapsuled( value, s );
 } // GenerateTZInfo
@@ -671,7 +724,7 @@ bool internalToVTIMEZONE( timecontext_t  aContext,
   }
   
   // at least one STANDARD or DAYLIGHT info is mandatory
-  aText=    Property      (    VTZ_ID,      tzn ) +
+  aText=    Property      (    VTZ_ID,      escapeText( tzn ) ) +
             GenerateTZInfo( t, VTZ_STD, id, t.std, y_std, t_plus,t.bias, g, aLogP );
 
   if (withDST)
@@ -855,17 +908,42 @@ bool ContextToTzDaylight( timecontext_t  aContext,
 
       timecontext_t                           cc= TCTX_UNKNOWN;
       while (FoundTZ( tCopy, s, cc, g, false, cc )) {
-        if (s.find( "/",0 )!=string::npos) { // search for a time zone with slash in it
+        // search for a time zone with slash in it (which is
+        // interpreted as separator between dstName and stdName) *or*
+        // one which has dstName and stdName set explicitly;
+        // prefer explicit names over splitting name
+        tz_entry zone;
+        if (GetTZ( cc, zone, g, -1 ) &&
+            !zone.stdName.empty() &&
+            !zone.dstName.empty()) {
+          s = zone.stdName + ";" + zone.dstName;
+          found= true; break;
+        } else if (s.find( "/",0 )!=string::npos) {
+          // Assumption here is that s contains exactly one slash,
+          // otherwise s is not valid for stdName;dstName in vCalendar
+          // 1.0.
+          StringSubst( s, "/", ";" );
           found= true; break;
         } // if
       } // while
 
       if (!found) {
-        s = t.name; // create a <x>;<x> string
+        string stdName;
+        if (t.stdName.empty()) {
+          stdName = t.name;
+        } else {
+          stdName = t.stdName;
+        }
+        s = stdName; // create a <x>;<x> string
         s+= ";";
-        s+= t.name;
+        s+= stdName;
       } // if
-    } // if
+    } else if (!t.stdName.empty() && !t.dstName.empty()) {
+      // use explicit zone names instead of splitting s
+      s = t.stdName + ";" + t.dstName;
+    } else {
+      StringSubst( s, "/", ";" );
+    }
 
     if (!dDone) dt= DST_Switch( t, t.bias, year, true  ); // get the switch time/date for DST
     if (!sDone) st= DST_Switch( t, t.bias, year, false ); // get the switch time/date for STD
@@ -892,7 +970,7 @@ bool ContextToTzDaylight( timecontext_t  aContext,
   string                 stdStr;
   TimestampToISO8601Str( stdStr, st, TCTX_UTC );
 
-  StringSubst                         ( s, "/", ";" );
+  // s must be of the format stdName;dstName here, for example CET;CEST or PST;PDT
   aText+= dstStr + ";" + stdStr + ";" + s;
 
   return true;
