@@ -29,14 +29,6 @@
 #include "sync_include.h"
 #include "sync_declarations.h"
 
-#ifdef JNI_SUPPORT
-  #ifdef MACOSX
-    #include <JavaVM/jni.h>
-  #else
-    #include <jni.h>
-  #endif
-#endif
-
 /* export prefix and suffix is dependent on platform and source language */
 #ifdef __cplusplus
   #define ENTRY_C extern "C"
@@ -46,6 +38,7 @@
 
 #if defined MACOSX
 	#define ENGINE_ENTRY ENTRY_C
+	#define ENGINE_ENTRY_CXX
 	
 	#ifdef __GNUC__
 	  #define ENTRY_ATTR __attribute__((visibility("default")))
@@ -56,14 +49,18 @@
   /* Visual Studio 2005 requires a specific entry point definition */
   /* This definition is empty for all other platforms */
   #define ENGINE_ENTRY ENTRY_C _declspec(dllexport)
+  #define ENGINE_ENTRY_CXX ENGINE_ENTRY
   #define ENTRY_ATTR
 #else
 	#define ENGINE_ENTRY ENTRY_C
+	#define ENGINE_ENTRY_CXX
 	#define ENTRY_ATTR
 #endif
 
 /* compose name of external symbols with C binding:
- * by default use parameter as-is (backwards compatibility),
+ * by default use parameter as-is (backwards compatibility) for client,
+ * and use "SySync_srv_" prefix for server.
+ * Newer builds usually use "SySync_" as prefix for client entry points.
  * can be changed in any file that is included before
  * this file
  */
@@ -71,6 +68,10 @@
 # define SYSYNC_EXTERNAL(_x) _x
 # define SYSYNC_PREFIX ""
 #endif
+#ifndef SYSYNC_EXTERNAL_SRV
+# define SYSYNC_EXTERNAL_SRV(_x) SySync_srv_ ## _x
+# define SYSYNC_PREFIX_SRV "SySync_srv_"
+#endif
 
 #ifdef __cplusplus
   namespace sysync {
@@ -92,11 +93,9 @@ enum Version {
 /*VP_Init                  0x01000000   * V1.0.N.0 : Initial version */
 /*VP_1st                   0x01000002   * V1.0.N.2 : 1st delivered version */
 /*VP_Session_Login         0x01000003   * V1.0.N.3 : VAR_String for Session_Login: this version only */
+/*VP_EngineVersionParam    0x01000004   * V1.0.N.4 : <engineVersion> param for "Module_PluginParams" */
+/*VP_CB_Version2         = 0x01000005   * V1.0.N.5 : Callback version >= 2 supported */
 
-  /** V1.0.N.4 : <engineVersion> param for "Module_PluginParams" */
-  VP_EngineVersionParam  = 0x01000004,
-  /** V1.0.N.5 : Callback version >= 2 supported                 */
-  VP_CB_Version2         = 0x01000005,
   /** V1.0.N.6 : With new function "DeleteBlob"                  */
   VP_DeleteBlob          = 0x01000006,
   /** V1.0.N.7 : With new "scripting" function "AdaptItemData"   */
@@ -152,8 +151,13 @@ enum Version {
   VP_CB_Version11        = 0x01050200,
   /** V1.6.0.X : Tunnel support                                  */
   VP_Tunnel              = 0x01060000,
-  /** V1.6.0.X : Current version, use 'Plugin_Version()'         */
-  VP_CurrentVersion      = 0x01060000,
+  /** V1.6.1.X : Correct SetValue support                        */
+  VP_SetValue            = 0x01060100,
+  /** V1.6.2.X : 64 bit Java BLOB signature                      */
+  VP_BLOB_JSignature     = 0x01060200,
+/*VP_091221                0x01060200   * V1.6.2.X : Released 21-Dec-09 */
+  /** V1.7.0.X : Current version, use 'Plugin_Version()'         */
+  VP_CurrentVersion      = 0x01070000,
 
   /** -------- : Bad/undefined version                           */
   VP_BadVersion          = 0xffffffff,
@@ -247,10 +251,12 @@ enum Version {
 #define CA_JNI               "JNI"          /* Indicates, if plugin is based on JNI */
 #define CA_CSHARP            "C#"           /* Indicates, if plugin is based on C#  */
 #define CA_GUID              "GUID"         /* GUID */
+#define CA_Plugin            "PLUGIN"       /* Built-In plugin name */
 #define CA_GlobContext       "GlobContext"  /* The global context, if available */
 #define CA_ADMIN_Info        "ADMIN_Info"   /* Get ADMIN info as <name><SP>"ADMIN" with 'CreateContext' */
                                             /*   (supported for V1.3.7 and higher) */
-#define CA_ItemAsKey         "ITEM_AS_KEY"  /* Supports the ItemAsKey" mode */
+#define CA_ItemAsKey         "ITEM_AS_KEY"  /* Supports the AsKey" mode */
+#define CA_AdminAsKey        "ADMIN_AS_KEY" /* Supports the AsKey" mode for Load/SaveAdminData */
 #define CA_Error             "ERROR"        /* Capability error */
 
 /* Predefined identifiers */
@@ -280,13 +286,14 @@ enum Version {
 
 #define Plugin_UI            "plugin_ui"
 
+
 /* Compatibility to older versions */
-#define Plugin_Param_OLD     "plugin_param_OLD"
 #define Plugin_SE_Auth_OLD   "plugin_sessionauth_OLD"
 #define Plugin_DS_Data_OLD1  "plugin_datastore_OLD1"
 #define Plugin_DS_Data_OLD2  "plugin_datastore_OLD2"
 #define Plugin_DS_Admin_OLD  "plugin_datastoreadmin_OLD"
-#define Plugin_DS_Blob_OLD   "plugin_datablob_OLD"
+#define Plugin_DS_Blob_OLD1  "plugin_datablob_OLD1"
+#define Plugin_DS_Blob_OLD2  "plugin_datablob_OLD2"
 
 
 
@@ -371,6 +378,8 @@ enum DebugFlags {
   DBG_PLUGIN_EXOT  = 0x0004,
   /** direct printf calls for test */ 
   DBG_PLUGIN_DIRECT= 0x0008,
+  /** GetValue/SetValue debugging */
+  DBG_GET_SET_VALUE= 0x0010,
   /** Default mask: all bits set */
   DBG_PLUGIN_ALL   = 0xffff,
 };
@@ -633,7 +642,7 @@ typedef TSyError (*DisconnectEngine_Func)	( UI_Call_In  aCI );
 
 
 
-/* Entry point for connecting the SyncML engine from outside */
+/* Entry point for connecting the SyncML engine as client from outside */
  ENGINE_ENTRY TSyError SYSYNC_EXTERNAL(ConnectEngine)
                                           ( UI_Call_In *aCI,
                                             CVersion   *aEngVersion,
@@ -647,10 +656,30 @@ typedef TSyError (*DisconnectEngine_Func)	( UI_Call_In  aCI );
                                             CVersion    aPrgVersion,
                                             uInt16      aDebugFlags ) ENTRY_ATTR;
 
-/* Entry point for disconnecting the engine at the end */
+/* Entry point for connecting the SyncML engine as server from outside */
+ ENGINE_ENTRY TSyError SYSYNC_EXTERNAL_SRV(ConnectEngine)
+                                          ( UI_Call_In *aCI,
+                                            CVersion   *aEngVersion,
+                                            CVersion    aPrgVersion,
+                                            uInt16      aDebugFlags ) ENTRY_ATTR;
+
+ ENGINE_ENTRY TSyError SYSYNC_EXTERNAL_SRV(ConnectEngineS)
+                                          ( UI_Call_In  aCI,
+                                            uInt16      aCallbackVersion,
+                                            CVersion   *aEngVersion,
+                                            CVersion    aPrgVersion,
+                                            uInt16      aDebugFlags ) ENTRY_ATTR;
+
+
+/* Entry point for disconnecting the client engine at the end */
  ENGINE_ENTRY TSyError SYSYNC_EXTERNAL(DisconnectEngine)
                                           ( UI_Call_In  aCI         ) ENTRY_ATTR;
 
+/* Entry point for disconnecting the client engine at the end */
+ ENGINE_ENTRY TSyError SYSYNC_EXTERNAL_SRV(DisconnectEngine)
+                                          ( UI_Call_In  aCI         ) ENTRY_ATTR;
+
+
 
 #ifdef __cplusplus
   } // namespace
