@@ -84,7 +84,6 @@ EvolutionSyncSource::Databases EvolutionContactSource::getDatabases()
 
     Databases secondary;
     Databases result;
-    bool first = true;
     for (GSList *g = e_source_list_peek_groups (sources); g; g = g->next) {
         ESourceGroup *group = E_SOURCE_GROUP (g->data);
         for (GSList *s = e_source_group_peek_sources (group); s; s = s->next) {
@@ -96,7 +95,7 @@ EvolutionSyncSource::Databases EvolutionContactSource::getDatabases()
             }
             Database entry(e_source_peek_name(source),
                            uristr,
-                           first);
+                           false);
             if (boost::starts_with(uristr, "couchdb://")) {
                 // Append CouchDB address books at the end of the list,
                 // otherwise preserving the order of address books.
@@ -115,7 +114,6 @@ EvolutionSyncSource::Databases EvolutionContactSource::getDatabases()
             } else {
                 result.push_back(entry);
             }
-            first = false;
         }
     }
     result.insert(result.end(), secondary.begin(), secondary.end());
@@ -139,6 +137,9 @@ EvolutionSyncSource::Databases EvolutionContactSource::getDatabases()
             const char *uri = e_book_get_uri (book);
             result.push_back(Database(name, uri, true));
         }
+    } else {
+        //  the first DB found is the default
+        result[0].m_isDefault = true;
     }
     
     return result;
@@ -155,6 +156,7 @@ void EvolutionContactSource::open()
     string id = getDatabaseID();
     ESource *source = findSource(sources, id);
     bool onlyIfExists = true;
+    bool created = false;
     if (!source) {
         // might have been special "<<system>>" or "<<default>>", try that and
         // creating address book from file:// URI before giving up
@@ -167,17 +169,22 @@ void EvolutionContactSource::open()
         } else {
             throwError(string(getName()) + ": no such address book: '" + id + "'");
         }
+        created = true;
         onlyIfExists = false;
     } else {
         m_addressbook.set( e_book_new( source, &gerror ), "address book" );
     }
  
     if (!e_book_open( m_addressbook, onlyIfExists, &gerror) ) {
-        // opening newly created address books often fails, try again once more
-        g_clear_error(&gerror);
-        sleep(5);
-        if (!e_book_open( m_addressbook, onlyIfExists, &gerror) ) {
-            throwError( "opening address book", gerror );
+        if (created) {
+            // opening newly created address books often fails, try again once more
+            g_clear_error(&gerror);
+            sleep(5);
+            if (!e_book_open(m_addressbook, onlyIfExists, &gerror)) {
+                throwError("opening address book", gerror);
+            }
+        } else {
+            throwError("opening address book", gerror);
         }
     }
 
@@ -218,6 +225,14 @@ void EvolutionContactSource::open()
                            (void *)"Evolution Data Server has died unexpectedly, contacts no longer available.");
 }
 
+bool EvolutionContactSource::isEmpty()
+{
+    // TODO: add more efficient implementation which does not
+    // depend on actually pulling all items from EDS
+    RevisionMap_t revisions;
+    listAllItems(revisions);
+    return revisions.empty();
+}
 
 void EvolutionContactSource::listAllItems(RevisionMap_t &revisions)
 {
@@ -287,6 +302,10 @@ void EvolutionContactSource::readItem(const string &luid, std::string &item, boo
         throwError(string("reading contact ") + luid,
                    gerror);
     }
+#ifdef ENABLE_MAEMO
+    // load detached photos etc
+    e_contact_inline_data(contact);
+#endif
     eptr<EContact, GObject> contactptr(contact, "contact");
     eptr<char> vcardstr(e_vcard_to_string(&contactptr->parent,
                                           EVC_FORMAT_VCARD_30));
