@@ -72,7 +72,7 @@ static RegisterSyncSource registerMe("Files in one directory",
                                      "      file:text/plain:1.0\n"
                                      "      file:text/x-vcard:2.1\n"
                                      "      file:text/vcard:3.0\n"
-                                     "      file:text/x-calendar:1.0\n"
+                                     "      file:text/x-vcalendar:1.0\n"
                                      "      file:text/calendar:2.0\n"
                                      "   Examples for evolutionsource:\n"
                                      "      /home/joe/datadir - directory must exist\n"
@@ -171,6 +171,18 @@ public:
     }
 } ITodo20Test;
 
+static class SuperTest : public RegisterSyncSourceTest {
+public:
+    SuperTest() : RegisterSyncSourceTest("file_calendar+todo", "calendar+todo") {}
+
+    virtual void updateConfig(ClientTestConfig &config) const
+    {
+        config.type = "virtual:text/x-vcalendar";
+        config.subConfigs = "file_ical20,file_itodo20";
+    }
+
+} superTest;
+
 }
 
 #endif // ENABLE_FILE
