@@ -103,6 +103,17 @@ void MaemoCalendarSource::open()
     conv->setSyncing(true); // not sure what this does, but may as well tell the truth
 }
 
+bool MaemoCalendarSource::isEmpty()
+{
+    int id = cal->getCalendarId(), err;
+    switch (entry_type) {
+    case EVENT:   return !mc->getEventCount(id, err);
+    case TODO:    return !mc->getTodoCount(id, err);
+    case JOURNAL: return !mc->getNoteCount(id, err);
+    default:      return true;
+    }
+}
+
 void MaemoCalendarSource::close()
 {
     delete conv;
@@ -272,6 +283,12 @@ TrackingSyncSource::InsertItemResult MaemoCalendarSource::insertItem(const strin
 void MaemoCalendarSource::removeItem(const string &uid)
 {
     int err;
+
+    if (cal->getCalendarType() == BIRTHDAY_CALENDAR) {
+        // stubbornly refuse to try this
+        throwError(string("can't sync smart calendar ") + cal->getCalendarName());
+    }
+
     cal->deleteComponent(uid, err);
     if (err != CALENDAR_OPERATION_SUCCESSFUL) {
         throwError(string("deleting item: ") + uid);
