@@ -153,6 +153,10 @@ void CurlTransportAgent::setCallback (TransportCallback cb, void *udata, int int
     m_cbInterval = interval;
 }
 
+void CurlTransportAgent::shutdown()
+{
+}
+
 void CurlTransportAgent::send(const char *data, size_t len)
 {
     CURLcode code;
@@ -203,7 +207,7 @@ void CurlTransportAgent::cancel()
     /* nothing to do */
 }
 
-TransportAgent::Status CurlTransportAgent::wait()
+TransportAgent::Status CurlTransportAgent::wait(bool noReply)
 {
     return m_status;
 }
@@ -279,7 +283,7 @@ void CurlTransportAgent::checkCurl(CURLcode code, bool exception)
 int CurlTransportAgent::progressCallback(void* transport, double, double, double, double)
 {
     CurlTransportAgent *agent = static_cast<CurlTransportAgent *> (transport);
-    SuspendFlags& s_flags = SyncContext::getSuspendFlags();
+    const SuspendFlags &s_flags = SyncContext::getSuspendFlags();
     //abort transfer
     if (s_flags.state == SuspendFlags::CLIENT_ABORT){
         agent->setAborting (true);
