@@ -21,6 +21,7 @@
 #include "config.h"
 #include <syncevo/LogRedirect.h>
 #include <syncevo/Logging.h>
+#include <syncevo/SyncContext.h>
 #include "test.h"
 #include <syncevo/util.h>
 #include <sys/types.h>
@@ -49,7 +50,8 @@ LogRedirect *LogRedirect::m_redirect;
 
 void LogRedirect::abortHandler(int sig) throw()
 {
-    SE_LOG_ERROR(NULL, NULL, "caught signal %d, shutting down", sig);
+    // Don't know state of logging system, don't log here!
+    // SE_LOG_ERROR(NULL, NULL, "caught signal %d, shutting down", sig);
 
     // shut down redirection, also flushes to log
     if (m_redirect) {
@@ -62,12 +64,13 @@ void LogRedirect::abortHandler(int sig) throw()
     raise(sig);
 }
 
-LogRedirect::LogRedirect(bool both) throw()
+void LogRedirect::init()
 {
     m_processing = false;
     m_buffer = NULL;
     m_len = 0;
     m_out = NULL;
+    m_streams = false;
     m_stderr.m_original =
         m_stderr.m_read =
         m_stderr.m_write =
@@ -76,6 +79,11 @@ LogRedirect::LogRedirect(bool both) throw()
         m_stdout.m_read =
         m_stdout.m_write =
         m_stdout.m_copy = -1;
+}
+
+LogRedirect::LogRedirect(bool both) throw()
+{
+    init();
     if (!getenv("SYNCEVOLUTION_DEBUG")) {
         redirect(STDERR_FILENO, m_stderr);
         if (both) {
@@ -109,9 +117,26 @@ LogRedirect::LogRedirect(bool both) throw()
     }
 }
 
+LogRedirect::LogRedirect(ExecuteFlags flags)
+{
+    init();
+
+    m_streams = true;
+    if (!(flags & EXECUTE_NO_STDERR)) {
+        redirect(STDERR_FILENO, m_stderr);
+    }
+    if (!(flags & EXECUTE_NO_STDOUT)) {
+        redirect(STDOUT_FILENO, m_stdout);
+    }
+}
+
 LogRedirect::~LogRedirect() throw()
 {
-    m_redirect = NULL;
+    bool pop = false;
+    if (m_redirect == this) {
+        m_redirect = NULL;
+        pop = true;
+    }
     process();
     restore();
     if (m_out) {
@@ -120,7 +145,9 @@ LogRedirect::~LogRedirect() throw()
     if (m_buffer) {
         free(m_buffer);
     }
-    LoggerBase::popLogger();
+    if (pop) {
+        LoggerBase::popLogger();
+    }
 }
 
 void LogRedirect::restore() throw()
@@ -153,57 +180,66 @@ void LogRedirect::redirect(int original, FDs &fds) throw()
     fds.m_write = fds.m_read = -1;
     fds.m_copy = dup(fds.m_original);
     if (fds.m_copy >= 0) {
-#ifdef USE_LOGREDIRECT_UNIX_DOMAIN
-        int sockets[2];
-
-        if (!socketpair(AF_LOCAL, SOCK_DGRAM, 0, sockets)) {
-            if (dup2(sockets[0], fds.m_original) >= 0) {
+        if (m_streams) {
+            // According to Stevens, Unix Network Programming, "Unix
+            // domain datagram sockets are similar to UDP sockets: the
+            // provide an *unreliable* datagram service that preserves
+            // record boundaries." (14.4 Socket Functions,
+            // p. 378). But unit tests showed that they do block on
+            // Linux and thus seem reliable. Not sure what the official
+            // spec is.
+            //
+            // To avoid the deadlock risk, we must use UDP. But when we
+            // want "reliable" behavior *and* detect that all output was
+            // processed, we have to use streams despite loosing
+            // the write() boundaries, because Unix domain datagram sockets
+            // do not flag "end of data".
+            int sockets[2];
+#define USE_UNIX_DOMAIN_DGRAM 0
+            if (!socketpair(AF_LOCAL,
+                            USE_UNIX_DOMAIN_DGRAM ? SOCK_DGRAM : SOCK_STREAM,
+                            0, sockets)) {
                 // success
                 fds.m_write = sockets[0];
                 fds.m_read = sockets[1];
                 return;
             } else {
-                perror("LogRedirect::redirect() dup2");
+                perror("LogRedirect::redirect() socketpair");
             }
-            close(sockets[0]);
-            close(sockets[1]);
         } else {
-            perror("LogRedirect::redirect() socketpair");
-        }
-#else
-        int write = socket(AF_INET, SOCK_DGRAM, 0);
-        if (write >= 0) {
-            int read = socket(AF_INET, SOCK_DGRAM, 0);
-            if (read >= 0) {
-                struct sockaddr_in addr;
-                memset(&addr, 0, sizeof(addr));
-                addr.sin_family = AF_INET;
-                addr.sin_addr.s_addr = inet_addr("127.0.0.1");
-                bool bound = false;
-                for (int port = 1025; !bound && port < 10000; port++) {
-                    addr.sin_port = htons(port);
-                    if (!bind(read, (struct sockaddr *)&addr, sizeof(addr))) {
-                        bound = true;
-                    }
-                }
-
-                if (bound) {
-                    if (!connect(write, (struct sockaddr *)&addr, sizeof(addr))) {
-                        if (dup2(write, fds.m_original) >= 0) {
-                            // success
-                            fds.m_write = write;
-                            fds.m_read = read;
-                            return;
+            int write = socket(AF_INET, SOCK_DGRAM, 0);
+            if (write >= 0) {
+                int read = socket(AF_INET, SOCK_DGRAM, 0);
+                if (read >= 0) {
+                    struct sockaddr_in addr;
+                    memset(&addr, 0, sizeof(addr));
+                    addr.sin_family = AF_INET;
+                    addr.sin_addr.s_addr = inet_addr("127.0.0.1");
+                    bool bound = false;
+                    for (int port = 1025; !bound && port < 10000; port++) {
+                        addr.sin_port = htons(port);
+                        if (!bind(read, (struct sockaddr *)&addr, sizeof(addr))) {
+                            bound = true;
                         }
-                        perror("LogRedirect::redirect() dup2");
                     }
-                    perror("LogRedirect::redirect connect");
+
+                    if (bound) {
+                        if (!connect(write, (struct sockaddr *)&addr, sizeof(addr))) {
+                            if (dup2(write, fds.m_original) >= 0) {
+                                // success
+                                fds.m_write = write;
+                                fds.m_read = read;
+                                return;
+                            }
+                            perror("LogRedirect::redirect() dup2");
+                        }
+                        perror("LogRedirect::redirect connect");
+                    }
+                    close(read);
                 }
-                close(read);
+                close(write);
             }
-            close(write);
         }
-#endif
         close(fds.m_copy);
         fds.m_copy = -1;
     } else {
@@ -213,35 +249,41 @@ void LogRedirect::redirect(int original, FDs &fds) throw()
 
 void LogRedirect::restore(FDs &fds) throw()
 {
-    if (fds.m_copy < 0) {
-        return;
+    if (!m_streams && fds.m_copy >= 0) {
+        // flush our own redirected output and process what they might have written
+        if (fds.m_original == STDOUT_FILENO) {
+            fflush(stdout);
+            std::cout << std::flush;
+        } else {
+            fflush(stderr);
+            std::cerr << std::flush;
+        }
+        process(fds);
+
+        dup2(fds.m_copy, fds.m_original);
     }
 
-    // flush streams and process what they might have written
-    if (fds.m_original == STDOUT_FILENO) {
-        fflush(stdout);
-        std::cout << std::flush;
-    } else {
-        fflush(stderr);
-        std::cerr << std::flush;
+    if (fds.m_copy >= 0) {
+        close(fds.m_copy);
+    }
+    if (fds.m_write >= 0) {
+        close(fds.m_write);
+    }
+    if (fds.m_read >= 0) {
+        close(fds.m_read);
     }
-    process(fds);
-
-    dup2(fds.m_copy, fds.m_original);
-    close(fds.m_copy);
-    close(fds.m_write);
-    close(fds.m_read);
     fds.m_copy =
         fds.m_write =
         fds.m_read = -1;
 }
 
-void LogRedirect::process(FDs &fds) throw()
+bool LogRedirect::process(FDs &fds) throw()
 {
     bool have_message;
+    bool data_read = false;
 
     if (fds.m_read <= 0) {
-        return;
+        return data_read;
     }
 
     do {
@@ -262,9 +304,30 @@ void LogRedirect::process(FDs &fds) throw()
                     m_len = newlen;
                 }
             }
-            // read, but leave space for nul byte
-            available = recv(fds.m_read, m_buffer, m_len - 1, MSG_PEEK|MSG_DONTWAIT);
-            have_message = available >= 0;
+            // read, but leave space for nul byte;
+            // when using datagrams, we only peek here and remove the
+            // datagram below, without rereading the data
+            if (!USE_UNIX_DOMAIN_DGRAM && m_streams) {
+                available = recv(fds.m_read, m_buffer, m_len - 1, MSG_DONTWAIT);
+                if (available == 0) {
+                    return data_read;
+                } else if (available == -1) {
+                    if (errno == EAGAIN) {
+                        // pretend that data was read, so that caller invokes us again
+                        return true;
+                    } else {
+                        SyncContext::throwError("reading output", errno);
+                        return false;
+                    }
+                } else {
+                    // data read, process it
+                    data_read = true;
+                    break;
+                }
+            } else {
+                available = recv(fds.m_read, m_buffer, m_len - 1, MSG_DONTWAIT|MSG_PEEK);
+                have_message = available >= 0;
+            }
             if (available < (ssize_t)m_len - 1) {
                 break;
             } else {
@@ -275,6 +338,7 @@ void LogRedirect::process(FDs &fds) throw()
         if (have_message) {
             // swallow packet, even if empty or we couldn't receive it
             recv(fds.m_read, NULL, 0, MSG_DONTWAIT);
+            data_read = true;
         }
 
         if (available > 0) {
@@ -284,11 +348,44 @@ void LogRedirect::process(FDs &fds) throw()
             // out known noise.
             const char *prefix = NULL;
             Logger::Level level = Logger::DEV;
-            const char *text = m_buffer;
+            char *text = m_buffer;
 
             if (fds.m_original == STDOUT_FILENO) {
                 // stdout: not sure what this could be, so show it
-                level = Logger::INFO;
+                level = Logger::SHOW;
+                char *eol = strchr(text, '\n');
+                if (!m_stdoutData.empty()) {
+                    // try to complete previous line, can be done
+                    // if text contains a line break
+                    if (eol) {
+                        m_stdoutData.append(text, eol - text);
+                        text = eol + 1;
+                        LoggerBase::instance().message(level, prefix,
+                                                       NULL, 0, NULL,
+                                                       "%s", m_stdoutData.c_str());
+                        m_stdoutData.clear();
+                    }
+                }
+
+                // avoid sending incomplete line at end of text,
+                // must be done when there is no line break or
+                // it is not at the end of the buffer
+                eol = strrchr(text, '\n');
+                if (eol != m_buffer + available - 1) {
+                    if (eol) {
+                        m_stdoutData.append(eol + 1);
+                        *eol = 0;
+                    } else {
+                        m_stdoutData.append(text);
+                        *text = 0;
+                    }
+                }
+
+                // output might have been processed as part of m_stdoutData,
+                // don't log empty string below
+                if (!*text) {
+                    continue;
+                }
             } else if (fds.m_original == STDERR_FILENO) {
                 // stderr: not normally useful for users, so we
                 // can filter it more aggressively. For example,
@@ -296,10 +393,11 @@ void LogRedirect::process(FDs &fds) throw()
                 while (*text == '\n') {
                     text++;
                 }
-                const char *glib_debug_prefix = "** (process:";
+                const char *glib_debug_prefix = "** ("; // ** (client-test:875): WARNING **:
                 const char *glib_msg_prefix = "** Message:";
                 prefix = "stderr";
-                if (!strncmp(text, glib_debug_prefix, strlen(glib_debug_prefix)) ||
+                if ((!strncmp(text, glib_debug_prefix, strlen(glib_debug_prefix)) &&
+                     strstr(text, " **:")) ||
                     !strncmp(text, glib_msg_prefix, strlen(glib_msg_prefix))) {
                     level = Logger::DEBUG;
                     prefix = "glib";
@@ -320,11 +418,80 @@ void LogRedirect::process(FDs &fds) throw()
                                            "%s", text);
         }
     } while(have_message);
+
+    return data_read;
 }
 
 
-void LogRedirect::process() throw()
+void LogRedirect::process()
 {
+    if (m_streams) {
+        // iterate until both sockets are closed by peer
+        while (true) {
+            fd_set readfds;
+            fd_set errfds;
+            int maxfd = 0;
+            FD_ZERO(&readfds);
+            FD_ZERO(&errfds);
+            if (m_stdout.m_read >= 0) {
+                FD_SET(m_stdout.m_read, &readfds);
+                FD_SET(m_stdout.m_read, &errfds);
+                maxfd = m_stdout.m_read;
+            }
+            if (m_stderr.m_read >= 0) {
+                FD_SET(m_stderr.m_read, &readfds);
+                FD_SET(m_stderr.m_read, &errfds);
+                if (m_stderr.m_read > maxfd) {
+                    maxfd = m_stderr.m_read;
+                }
+            }
+            if (maxfd == 0) {
+                // both closed
+                return;
+            }
+
+            int res = select(maxfd + 1, &readfds, NULL, &errfds, NULL);
+            switch (res) {
+            case -1:
+                // fatal, cannot continue
+                SyncContext::throwError("waiting for output", errno);
+                return;
+                break;
+            case 0:
+                // None ready? Try again.
+                break;
+            default:
+                if (m_stdout.m_read >= 0 && FD_ISSET(m_stdout.m_read, &readfds)) {
+                    if (!process(m_stdout)) {
+                        // Exact status of a Unix domain datagram socket upon close
+                        // of the remote end is a bit uncertain. For TCP, we would end
+                        // up here: marked by select as "ready for read", but no data -> EOF.
+                        close(m_stdout.m_read);
+                        m_stdout.m_read = -1;
+                    }
+                }
+                if (m_stdout.m_read >= 0 && FD_ISSET(m_stdout.m_read, &errfds)) {
+                    // But in practice, Unix domain sockets don't mark the stream
+                    // as "closed". This is an attempt to detect that situation
+                    // via the FDs exception status, but that also doesn't work.
+                    close(m_stdout.m_read);
+                    m_stdout.m_read = -1;
+                }
+                if (m_stderr.m_read >= 0 && FD_ISSET(m_stderr.m_read, &readfds)) {
+                    if (!process(m_stderr)) {
+                        close(m_stderr.m_read);
+                        m_stderr.m_read = -1;
+                    }
+                }
+                if (m_stderr.m_read >= 0 && FD_ISSET(m_stderr.m_read, &errfds)) {
+                    close(m_stderr.m_read);
+                    m_stderr.m_read = -1;
+                }
+                break;
+            }
+        }
+    }
+
     if (m_processing) {
         return;
     }
@@ -344,6 +511,19 @@ void LogRedirect::process() throw()
 }
 
 
+
+void LogRedirect::flush() throw()
+{
+    process();
+    if (!m_stdoutData.empty()) {
+        LoggerBase::instance().message(Logger::SHOW, NULL,
+                                       NULL, 0, NULL,
+                                       "%s", m_stdoutData.c_str());
+        m_stdoutData.clear();
+    }
+}
+
+
 #ifdef ENABLE_UNIT_TESTS
 
 class LogRedirectTest : public CppUnit::TestFixture {
@@ -398,9 +578,9 @@ public:
 
         static const char *simpleMessage = "hello world";
         CPPUNIT_ASSERT_EQUAL((ssize_t)strlen(simpleMessage), write(STDOUT_FILENO, simpleMessage, strlen(simpleMessage)));
-        buffer.m_redirect->process();
+        buffer.m_redirect->flush();
 
-        CPPUNIT_ASSERT_EQUAL(buffer.m_streams[Logger::INFO].str(), std::string(simpleMessage));
+        CPPUNIT_ASSERT_EQUAL(buffer.m_streams[Logger::SHOW].str(), std::string(simpleMessage));
     }
 
     void largeChunk()
@@ -410,10 +590,10 @@ public:
         std::string large;
         large.append(60 * 1024, 'h');
         CPPUNIT_ASSERT_EQUAL((ssize_t)large.size(), write(STDOUT_FILENO, large.c_str(), large.size()));
-        buffer.m_redirect->process();
+        buffer.m_redirect->flush();
 
-        CPPUNIT_ASSERT_EQUAL(large.size(), buffer.m_streams[Logger::INFO].str().size());
-        CPPUNIT_ASSERT_EQUAL(large, buffer.m_streams[Logger::INFO].str());
+        CPPUNIT_ASSERT_EQUAL(large.size(), buffer.m_streams[Logger::SHOW].str().size());
+        CPPUNIT_ASSERT_EQUAL(large, buffer.m_streams[Logger::SHOW].str());
     }
 
     void streams()
@@ -424,10 +604,16 @@ public:
         CPPUNIT_ASSERT_EQUAL((ssize_t)strlen(simpleMessage), write(STDOUT_FILENO, simpleMessage, strlen(simpleMessage)));
         static const char *errorMessage = "such a cruel place";
         CPPUNIT_ASSERT_EQUAL((ssize_t)strlen(errorMessage), write(STDERR_FILENO, errorMessage, strlen(errorMessage)));
-        buffer.m_redirect->process();
 
-        CPPUNIT_ASSERT_EQUAL(std::string(simpleMessage), buffer.m_streams[Logger::INFO].str());
+        // process() keeps unfinished STDOUT lines buffered
+        buffer.m_redirect->process();
         CPPUNIT_ASSERT_EQUAL(std::string(errorMessage), buffer.m_streams[Logger::DEV].str());
+        CPPUNIT_ASSERT_EQUAL(string(""), buffer.m_streams[Logger::SHOW].str());
+
+        // flush() makes them available
+        buffer.m_redirect->flush();
+        CPPUNIT_ASSERT_EQUAL(std::string(errorMessage), buffer.m_streams[Logger::DEV].str());
+        CPPUNIT_ASSERT_EQUAL(std::string(simpleMessage), buffer.m_streams[Logger::SHOW].str());
     }
 
     void overload()
@@ -439,9 +625,9 @@ public:
         for (int i = 0; i < 4000; i++) {
             CPPUNIT_ASSERT_EQUAL((ssize_t)large.size(), write(STDOUT_FILENO, large.c_str(), large.size()));
         }
-        buffer.m_redirect->process();
+        buffer.m_redirect->flush();
 
-        CPPUNIT_ASSERT(buffer.m_streams[Logger::INFO].str().size() > large.size());
+        CPPUNIT_ASSERT(buffer.m_streams[Logger::SHOW].str().size() > large.size());
     }
 
 #ifdef HAVE_GLIB
@@ -477,10 +663,19 @@ public:
 
             buffer.m_redirect->process();
 
-            std::string debug = buffer.m_streams[Logger::DEBUG].str();
+            std::string error = buffer.m_streams[Logger::ERROR].str();
+            std::string warning = buffer.m_streams[Logger::WARNING].str();           
+            std::string show = buffer.m_streams[Logger::SHOW].str();
+            std::string info = buffer.m_streams[Logger::INFO].str();
             std::string dev = buffer.m_streams[Logger::DEV].str();
-            CPPUNIT_ASSERT(debug.find("test warning") != debug.npos);
+            std::string debug = buffer.m_streams[Logger::DEBUG].str();
+            CPPUNIT_ASSERT_EQUAL(string(""), error);
+            CPPUNIT_ASSERT_EQUAL(string(""), warning);
+            CPPUNIT_ASSERT_EQUAL(string(""), show);
+            CPPUNIT_ASSERT_EQUAL(string(""), info);
+            CPPUNIT_ASSERT_EQUAL(string(""), error);
             CPPUNIT_ASSERT(dev.find("normal message stderr") != dev.npos);
+            CPPUNIT_ASSERT(debug.find("test warning") != debug.npos);
         } catch(...) {
             dup2(orig_stdout, STDOUT_FILENO);
             throw;
