@@ -294,7 +294,7 @@ struct ClientTestConfig{
      * both items independently. However, operations which violate
      * the semantic of the related items (like deleting the parent, but
      * not the child) may have unspecified results (like also deleting
-     * the child). See LINKED_ITEMS_RELAXED_SEMANTIC.
+     * the child). See linkedItemsRelaxedSemantic and sourceKnowsItemSemantic.
      *
      * One example for main and subordinate items are a recurring
      * iCalendar 2.0 event and a detached recurrence.
@@ -307,13 +307,11 @@ struct ClientTestConfig{
     bool atomicModification;
 
     /**
-     * define to 0 to disable tests which slightly violate the
+     * set to false to disable tests which slightly violate the
      * semantic of linked items by inserting children
      * before/without their parent
      */
-#ifndef LINKED_ITEMS_RELAXED_SEMANTIC
-# define LINKED_ITEMS_RELAXED_SEMANTIC 1
-#endif
+    bool linkedItemsRelaxedSemantic;
 
     /**
      * setting this to false disables tests which depend
