@@ -40,6 +40,7 @@
 
 #include <Logging.h>
 #include <LogStdout.h>
+#include "ClientTest.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -126,11 +127,13 @@ public:
     void startTest (CppUnit::Test *test) {
         m_currentTest = test->getName();
         std::cerr << m_currentTest;
-        string logfile = m_currentTest + ".log";
-        simplifyFilename(logfile);
-        m_logger.reset(new LoggerStdout(logfile));
-        m_logger->setLevel(Logger::DEBUG);
-        LoggerBase::pushLogger(m_logger.get());
+        if (!getenv("SYNCEVOLUTION_DEBUG")) {
+            string logfile = m_currentTest + ".log";
+            simplifyFilename(logfile);
+            m_logger.reset(new LoggerStdout(logfile));
+            m_logger->setLevel(Logger::DEBUG);
+            LoggerBase::pushLogger(m_logger.get());
+        }
         SE_LOG_DEBUG(NULL, NULL, "*** starting %s ***", m_currentTest.c_str());
         m_failures.reset();
         m_testFailed = false;
@@ -175,7 +178,9 @@ public:
         if (!failure.empty()) {
             SE_LOG_DEBUG(NULL, NULL, "%s", failure.c_str());
         }
-        LoggerBase::popLogger();
+        if (&LoggerBase::instance() == m_logger.get()) {
+            LoggerBase::popLogger();
+        }
         m_logger.reset();
 
         string logfile = m_currentTest + ".log";
@@ -260,6 +265,11 @@ int main(int argc, char* argv[])
   }
   runner.eventManager().addListener(&syncListener);
 
+
+  if (getenv("SYNCEVOLUTION_DEBUG")) {
+      LoggerBase::instance().setLevel(Logger::DEBUG);
+  }
+
   try {
       // Run the tests.
       if (argc <= 1) {
@@ -273,12 +283,15 @@ int main(int argc, char* argv[])
       }
 
       // Return error code 1 if the one of test failed.
+      ClientTest::shutdown();
       return syncListener.hasFailed() ? 1 : 0;
   } catch (invalid_argument e) {
       // Test path not resolved
       std::cerr << std::endl
                 << "ERROR: " << e.what()
                 << std::endl;
+
+      ClientTest::shutdown();
       return 1;
   }
 }
